==> saphire-3.0.1.tgz: saphire-3.0.1/saphire_inner.h <==
#if !defined(SAPHIRE_INNER_H)
#define SAPHIRE_INNER_H

////////////////////////////////////////////////////////////
/// from saphire_main.c
////////////////////////////////////////////////////////////
extern volatile BOOL gSigUser;

/// ޥɤбʸ
enum eCommandKind { 
    kVar, kGlobal, kExport, 
    kAry, kAryAdd, kAryErase, 
    kHash, kHashAdd, kHashErase,  
    kSubshell, kMSleep, kTrue, kFalse, kTest,
    kIndex, kRIndex, kLength, kLc, kUc,
    kChomp, kSubstr, kEval, kFg, kBg, 
    kJobs, kRehash, kKanjiCode, kLineField,
    kCalc, kPrint,  kPuts, kLoad, 
    kWhile, kBreak, kExit, kIf, kSplit, kX, kJoin, kSelect, kLines, kRows,
    kMatch, kScan, kErase, kSub,
    kRead, 
    kCd, 
    kAdd, kDel, kSelector, kMax, kMin,
    kExtName, kParentName, kNoExtName, 
    kCompile, kRaise,
    kPopd, kPushd,
    kPlusPlus,
    kMinusMinus,
    kPlus,
    kMinus,
    kMult,
    kDiv,
    kMod,
    kPow,
    kSort,
    kRange,
    kPrintf,
    kPomch,
    kDef,
    kPTee,
    kPCat,
    kReturn,
    kMInitScr,
    kMEndWin,
    kMClear,
    kMClearImmediately,
    kMRefresh,
    kMMove ,
    kMMoveImmediately,
    kMPrintw, 
    kSweep,
    kShow,
    kYeild,
    kEach,
    kForeach,
//    kParse,
    kAbs,
    kFAbs,
    kP,
    kTry,
    kInnerCommand,
    kCommand 
};

extern string_obj* gAncestalMemoryPipeIn;
    // ȥåץ٥ޥѤΥѥ
extern string_obj* gAncestalMemoryPipeOut;
    // ȥåץ٥ޥѤΥѥ

extern char gTmpDir[PATH_MAX];
    // ե֤

extern char* gStatmentKindStrs[kCommand+1];
extern hash_obj* gFuncs;
    // 桼ؿ sFunction*äƤ
    // ϴؿ̾

extern vector_obj* gStackFrame;
    // ѿΥå
    // Ƥhash_obj*
    // Υϥѿ̾
    // Ƥstring_obj*
extern enum eAppType gAppType;
    // ץꥱμ
extern BOOL gJobControl;
    // ֤Υȥ򤹤뤫ɤ
extern vector_obj* gJobs;
    //  sJob*

extern char* gMagicNumber;
    // ֥ȥեΥޥåʥС

extern hash_obj* gReadBuffers;
    //  ե̾
    //  sRFd*
    // readѤΥեΥ꡼ɥХåե

extern vector_obj* gGlobalPipes;
    // å֥륰Хѥפɤ߹ߤǡ

extern vector_obj* gQGlobalPipes;
    // 塼Хѥפɤ߹ߤǡ

extern string_obj* gGlobalPipe;
    // Хѥפɤ߹ߤǡ

extern string_obj* gGlobalPipeNum[10];
    // ʥСХѥפɤ߹ߤǡ
extern BOOL gSaphireBackground;
    // SaphireХå饦ɤưƤ뤫ɤ

extern hash_obj* gRegexs;
    // ɽΥå
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsI;
    // ɽΥå(ignore case)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsM;
    // ɽΥå(multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsIM;
    // ɽΥå(ignore case|multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gFReadBuffers;
    // ɤ߹ߥХåե
    // ϥեǥץֹʸ
    // Ƥɤ߹ե
    // statment_tree_internal_commands_read_nextin_oneline
    // ǻȤ
extern hash_obj* gFWriteBuffers;
    // 񤭹ߥХåե
    // ϥեǥץֹʸ
    // ƤϽ񤭹ե

extern hash_obj* gInnerCommands;
    // Ͽޥ
    // saphireȤ߹ץꥱ
    // 줿ޥɤΥǡ
    // ϥޥ̾
    // ƤsInnerCommand

extern vector_obj* gDirStack;
    // popd, pushdǻȤǥ쥯ȥꥹå
    // Ƥ string_obj*
extern enum eLineField gLineField;
    // saphireʸޥɤǻȤԥ

extern vector_obj* gPSubTmpFiles;
    // ץִǻȤեǾä
    // ¸Ƥ

BOOL is_line_field(char* p);
    // 饤եɤݥȤƤ뤫

BOOL is_line_field2(enum eLineField lf, char* p);
    // 饤եɤݥȤƤ뤫2

BOOL string_chomp(string_obj* str);
    // ԥɤ

void string_ponch(string_obj* str);
    // ԥɤդ(\n)

void string_pomch(string_obj* str, enum eLineField lf);
    // ԥɤդ(ԥɤΤޤ)

BOOL xclose(int fd);
    // closeȤˤΥեǥץ
    // gFReadBuffersΥХåեк
    // gFWriteBuffersΥХåեн񤭹Ǻ

int get_command_kind(char* arg0);
    // ޥ̾饳ޥIDõ

void null_fun();
    // gotoΥ٥򥳥ѥ顼̤δؿ

BOOL parse_is_redirect(char* p);
    // ݥ󥿡ʸ󤬥쥯Ȥɤ

BOOL parse_is_redirect2(char* p);
    // ݥ󥿤ʸ󤬥쥯Ȥɤ2

BOOL is_env_name_char(char c);
    // ѿǻȤʸɤ

BOOL is_glob_terminated_char(char* p);
    // ֤ζڤʸɤ

BOOL is_terminated_char(char* p);
    // 

void get_quoted_linefield(char* str, string_obj* out);

typedef struct _sJob {
    string_obj* mName;          // ֤̾
    pid_t mPGroup;              // ץ롼ID
    vector_obj* mPIDs;          // ҥץPIDΥꥹ
    struct termios* mTty;
} sJob;

sJob* sJob_new();
void sJob_delete(sJob* self);

enum eStatmentTerminated { kTNormal, kTOrOr, kTAndAnd };

typedef struct
{
    sStatments* joukensiki;
    sStatments* contents;
} sWhile;

sWhile* sWhile_new(MANAGED sStatments* joukensiki, MANAGED sStatments* contents);
sWhile* sWhile_new2(sWhile* while_);

void sWhile_delete(sWhile* self);
void sWhile_save(sWhile* self, int fd);
sWhile* sWhile_load(int fd);

typedef struct
{
    sStatments* loops;
    sStatments* loops2;
} sTry;

sTry* sTry_new(MANAGED sStatments* loops, MANAGED sStatments* loops2);
sTry* sTry_new2(sTry* try_);
void sTry_view(sTry* self);
void sTry_delete(sTry* self);
void sTry_save(sTry* self, int fd);
sTry* sTry_load(int fd);

typedef struct
{
    sStatments* contents;
} sSubshell;

sSubshell* sSubshell_new(MANAGED sStatments* contents);
sSubshell* sSubshell_new2(sSubshell* subshell);

void sSubshell_delete(sSubshell* self);
void sSubshell_save(sSubshell* self, int fd);
sSubshell* sSubshell_load(int fd);
void sSubshell_view(sSubshell* self);

typedef struct
{
    vector_obj* joukensiki_list;
    vector_obj* contents_list;
} sIf;

sIf* sIf_new(MANAGED vector_obj* joukensiki_list, MANAGED vector_obj* contents_list);
sIf* sIf_new2(sIf* if_);

void sIf_delete(sIf* self);
void sIf_save(sIf* self, int fd);
sIf* sIf_load(int fd);

enum eCommandKind2 {
    kCKLocalVar, kCKUserCommand, kCKExternInnerCommand, kCKInnerCommand,
    kCKExternCommand
};

enum eRedirect { kRedirectInput, kRedirectOverwrite, kRedirectAppend
          , kRedirectErrAndOutput };

typedef struct {
    enum eRedirect mType;
    int mFd;
    string_obj* mFName;
} sRedirect; // ¹Ի˻Ȥ ֥ȥե¸

sRedirect* sRedirect_new(enum eRedirect type, int fd, char* fname);
sRedirect* sRedirect_new2(sRedirect* redirect);
void sRedirect_save(sRedirect* self, int fd);
sRedirect* sRedirect_load(int fd);
void sRedirect_delete(sRedirect* self);

typedef struct {
    enum eRedirect mType;
    int mFd;
} sRedirectTmp; // ѡǰŪ˻Ȥ

sRedirectTmp* sRedirectTmp_new(enum eRedirect type, int fd);
void sRedirectTmp_delete(sRedirectTmp* self);

typedef struct {
    string_obj* mName;

    string_obj* mIndex;
    string_obj* mIndex2;
    string_obj* mDelimiter;
} sVar;

sVar* sVar_new(MANAGED string_obj* name,  MANAGED string_obj* index, MANAGED string_obj* index2, MANAGED string_obj* delimiter);
sVar* sVar_new2(sVar* var);
void sVar_delete(sVar* self);

sVar* sVar_load(int fd);
void sVar_save(sVar* self, int fd);

sBlock* sBlock_new(sStatments* statments, vector_obj* args, int count);
sBlock* sBlock_new2(sBlock* block);
void sBlock_delete(sBlock* self);
void sBlock_save(sBlock* self, int fd);
sBlock* sBlock_load(int fd);

typedef struct
{
    int mKind;   // 0: string 1: var 2: redirect 3: statments 4:var2 5: statments2 6: block
    void* mBody;
} sArgTmp; // ѡǰŪ˻Ȥ

sArgTmp* sArgTmp_new(int kind, void* body);
void sArgTmp_delete(sArgTmp* arg);

typedef struct
{
    int mKind; // 0:string 1: var 2: statments 3:var2 4: statments2 5: block
    void* mBody;
} sArg; // mArgs

sArg* sArg_new(int kind, void* body);
sArg* sArg_new2(sArg* arg);
void sArg_delete(sArg* arg);

void sArg_save(sArg* self, int fd);
sArg* sArg_load(int fd);

typedef struct
{
    enum eCommandKind mKind;
    vector_obj* mArgs;
        // vector of sArg*
        // ޥɥ饤
    vector_obj* mRedirects;
        // vector of sRedirect* 
        // 쥯Υꥹ

    void* mExtra;

    vector_obj* mArgTmps;
        // vector of sArgTmp*
        // ѡѤ˰Ū˻Ȥѿ
} sCommand;

#ifdef MDEBUG 
void sCommand_view(sCommand* self);
#endif

sCommand* sCommand_new();
#define COMMAND_NEW() sCommand_new()

void sCommand_save(sCommand* self, int fd);
sCommand* sCommand_load(int fd);
void sCommand_delete(sCommand* self);

typedef struct
{
    vector_obj* mCommands;   // ޥ

    string_obj* mFName;      // ʸե̾
    int mLine;               // ʸ

    string_obj* mNotEvaled;  // run˥ѡʸ
                             // Ķѿȥޥޤʸ
    string_obj* mTitle;      // ȥ

    enum eStatmentTerminated mTerminated;   
            // ʣʸκǸ夬ǶڤƤ뤫

    BOOL mRCodeReverse;      // ꥿󥳡ɤȿž

    BOOL mBackground;        // Хå饦ɤǼ¹

    int mGlobalPipeIn;   // ХѥפϤ뤫 1:  2: å֥ 3:塼 4: ʥС
    int mGlobalPipeOut;  // ХѥפνϤ뤫 1:  2: å֥ 3:塼 4:ʥС
    int mGlobalPipeInNum; // ʥСХѥפֹ
    int mGlobalPipeOutNum; // ʥСХѥפֹ
} sStatment;

#ifdef MDEBUG
void sStatment_view(sStatment* self);
#endif

sStatment* sStatment_new();
#define STATMENT_NEW() sStatment_new()

sStatment* sStatment_new2(sStatment* statment);

void sStatment_delete(sStatment* self);
void sStatment_save(sStatment* self, int fd);
sStatment* sStatment_load(int fd);

sStatments* sStatments_new();
#define STATMENTS_NEW() sStatments_new()

void sStatments_view(sStatments* self);

void sStatments_save(sStatments* self, int fd);
sStatments* sStatments_load(int fd);
void sStatments_delete(sStatments* self);

typedef struct {
    string_obj* name;       // ޥ̾
    string_obj* arg_name;
    sStatments* statments; // ̿ɽ
} sFunction;

sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_name);
sFunction* sFunction_new2(sFunction* func);
  
void sFunction_delete(sFunction* self);
void sFunction_save(sFunction* self, int fd);
sFunction* sFunction_load(int fd);
void sFunction_view(sFunction* self);

typedef struct
{
    int mPid;
    int mStatus;
    int mPGroup;
} sStrayCProg;

extern vector_obj* gStrayCProgs;

sStrayCProg* sStrayCprog_new(int pid, int status, int pgroup);
void sStrayCProg_delete(sStrayCProg* self);

typedef struct {
    pid_t mPid;
    string_obj* mTitle;
    BOOL mWriterProcess;
} sChildProgram;

sChildProgram* sChildProgram_new(pid_t pid, char* title, BOOL writer_process);
void sChildProgram_delete(sChildProgram* self);

typedef struct {
    string_obj* mName;
    fInnerCommand mFun;
} sInnerCommand;

sInnerCommand* sInnerCommand_new(char* name, fInnerCommand fun);
void sInnerCommand_delete(sInnerCommand* self);

void sigchld_block(int block);

////////////////////////////////////////////////////////////
/// from saphire_commands.c
////////////////////////////////////////////////////////////
int statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, int j, char* title, BOOL input, BOOL* return_, char* sname, int line);
BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers,char* sname, int line);

////////////////////////////////////////////////////////////
/// from saphire_parser.c
////////////////////////////////////////////////////////////
BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* line);
BOOL parse(char* cmdline, char* sname, int* line, sStatments* statments);
BOOL read_expand_brace_expansion(char* p, string_obj* cmdline2, char* separator, char* sname, int* line);
BOOL parse_number(char* p, vector_obj* numbers, char* sname, int line);

typedef struct {
    int number;
    int number2;
    int number_num;
} sIndex;

sIndex* sIndex_new(int num, int num2, int num_num);
void sIndex_delete(sIndex* self);

BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment, BOOL line_inc, char* sname, int* line);

////////////////////////////////////////////////////////////
// from saphire_vm.c
////////////////////////////////////////////////////////////
BOOL forground_job(int num);
void background_job(int num);
void tmpfile_all_sweep();
void tmpfile_sweep();

extern vector_obj* gStackTraceFName;
extern vector_obj* gStackTraceLineNum;
    // åե졼εϿ

int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_);
    // saphire_compile2ǥѥ뤷ѥ̤¹Ԥ롣
    // verboseϼ¹Ԥ˼¹Ԥ륳ޥ̾ɽ뤫ɤ

#endif

==> saphire-3.0.7.tgz: saphire-3.0.7/saphire_inner.h <==
#if !defined(SAPHIRE_INNER_H)
#define SAPHIRE_INNER_H

////////////////////////////////////////////////////////////
/// from saphire_main.c
////////////////////////////////////////////////////////////
extern volatile BOOL gSigUser;

/// ޥɤбʸ
enum eCommandKind { 
    kVar, kGlobal, kExport, 
    kAry, kAryAdd, kAryErase, 
    kHash, kHashAdd, kHashErase,  
    kSubshell, kMSleep, kTrue, kFalse, kTest,
    kIndex, kRIndex, kLength, kLc, kUc,
    kChomp, kSubstr, kEval, kFg, kBg, 
    kJobs, kRehash, kKanjiCode, kLineField,
    kCalc, kPrint,  kPuts, kLoad, 
    kWhile, kBreak, kExit, kIf, kSplit, kX, kJoin, kSelect, kLines, kRows,
    kMatch, kScan, kErase, kSub,
    kRead, 
    kCd, 
    kAdd, kDel, kSelector, kMax, kMin,
    kExtName, kParentName, kNoExtName, 
    kCompile, kRaise,
    kPopd, kPushd,
    kPlusPlus,
    kMinusMinus,
    kPlus,
    kMinus,
    kMult,
    kDiv,
    kMod,
    kPow,
    kSort,
    kRange,
    kPrintf,
    kPomch,
    kDef,
    kPTee,
    kPCat,
    kReturn,
    kMInitScr,
    kMEndWin,
    kMClear,
    kMClearImmediately,
    kMRefresh,
    kMMove ,
    kMMoveImmediately,
    kMPrintw, 
    kSweep,
    kShow,
    kYeild,
    kEach,
    kForeach,
//    kParse,
    kAbs,
    kFAbs,
    kP,
    kTry,
    kInnerCommand,
    kCommand 
};

extern char gTmpDir[PATH_MAX];
    // ե֤

extern char* gStatmentKindStrs[kCommand+1];
extern hash_obj* gFuncs;
    // 桼ؿ sFunction*äƤ
    // ϴؿ̾

extern vector_obj* gStackFrame;
    // ѿΥå
    // Ƥhash_obj*
    // Υϥѿ̾
    // Ƥstring_obj*
extern enum eAppType gAppType;
    // ץꥱμ
extern BOOL gJobControl;
    // ֤Υȥ򤹤뤫ɤ
extern vector_obj* gJobs;
    //  sJob*

extern char* gMagicNumber;
    // ֥ȥեΥޥåʥС

extern hash_obj* gReadBuffers;
    //  ե̾
    //  sRFd*
    // readѤΥեΥ꡼ɥХåե

extern string_obj* gGlobalPipe;
    // Хѥפɤ߹ߤǡ

extern string_obj* gGlobalPipeNum[10];
    // ʥСХѥפɤ߹ߤǡ
extern BOOL gSaphireBackground;
    // SaphireХå饦ɤưƤ뤫ɤ

extern hash_obj* gRegexs;
    // ɽΥå
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsI;
    // ɽΥå(ignore case)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsM;
    // ɽΥå(multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gRegexsIM;
    // ɽΥå(ignore case|multi line)
    // ɽʸ
    // Ƥregex_t

extern hash_obj* gInnerCommands;
    // Ͽޥ
    // saphireȤ߹ץꥱ
    // 줿ޥɤΥǡ
    // ϥޥ̾
    // ƤsInnerCommand

extern vector_obj* gDirStack;
    // popd, pushdǻȤǥ쥯ȥꥹå
    // Ƥ string_obj*
extern enum eLineField gLineField;
    // saphireʸޥɤǻȤԥ

extern vector_obj* gPSubTmpFiles;
    // ץִǻȤեǾä
    // ¸Ƥ

BOOL is_line_field(char* p);
    // 饤եɤݥȤƤ뤫

BOOL is_line_field2(enum eLineField lf, char* p);
    // 饤եɤݥȤƤ뤫2

BOOL string_chomp(string_obj* str);
    // ԥɤ

void string_ponch(string_obj* str);
    // ԥɤդ(\n)

void string_pomch(string_obj* str, enum eLineField lf);
    // ԥɤդ(ԥɤΤޤ)

int get_command_kind(char* arg0);
    // ޥ̾饳ޥIDõ

void null_fun();
    // gotoΥ٥򥳥ѥ顼̤δؿ

void get_quoted_linefield(char* str, string_obj* out);

typedef struct _sJob {
    string_obj* mName;          // ֤̾
    pid_t mPGroup;              // ץ롼ID
    vector_obj* mPIDs;          // ҥץPIDΥꥹ
    struct termios* mTty;
} sJob;

sJob* sJob_new();
void sJob_delete(sJob* self);

enum eStatmentTerminated { kTNormal, kTOrOr, kTAndAnd };

typedef struct
{
    sStatments* joukensiki;
    sStatments* contents;
} sWhile;

sWhile* sWhile_new(MANAGED sStatments* joukensiki, MANAGED sStatments* contents);
sWhile* sWhile_new2(sWhile* while_);

void sWhile_delete(sWhile* self);
void sWhile_save(sWhile* self, int fd);
sWhile* sWhile_load(int fd);

typedef struct
{
    sStatments* loops;
    sStatments* loops2;
} sTry;

sTry* sTry_new(MANAGED sStatments* loops, MANAGED sStatments* loops2);
sTry* sTry_new2(sTry* try_);
void sTry_view(sTry* self);
void sTry_delete(sTry* self);
void sTry_save(sTry* self, int fd);
sTry* sTry_load(int fd);

typedef struct
{
    sStatments* contents;
} sSubshell;

sSubshell* sSubshell_new(MANAGED sStatments* contents);
sSubshell* sSubshell_new2(sSubshell* subshell);

void sSubshell_delete(sSubshell* self);
void sSubshell_save(sSubshell* self, int fd);
sSubshell* sSubshell_load(int fd);
void sSubshell_view(sSubshell* self);

typedef struct
{
    vector_obj* joukensiki_list;
    vector_obj* contents_list;
} sIf;

sIf* sIf_new(MANAGED vector_obj* joukensiki_list, MANAGED vector_obj* contents_list);
sIf* sIf_new2(sIf* if_);

void sIf_delete(sIf* self);
void sIf_save(sIf* self, int fd);
sIf* sIf_load(int fd);

enum eCommandKind2 {
    kCKLocalVar, kCKUserCommand, kCKExternInnerCommand, kCKInnerCommand,
    kCKExternCommand
};

enum eRedirect { kRedirectInput, kRedirectOverwrite, kRedirectAppend
          , kRedirectErrAndOutput };

typedef struct {
    enum eRedirect mType;
    int mFd;
    string_obj* mFName;
} sRedirect; // ¹Ի˻Ȥ ֥ȥե¸

sRedirect* sRedirect_new(enum eRedirect type, int fd, char* fname);
sRedirect* sRedirect_new2(sRedirect* redirect);
void sRedirect_save(sRedirect* self, int fd);
sRedirect* sRedirect_load(int fd);
void sRedirect_delete(sRedirect* self);

sBlock* sBlock_new(sStatments* statments, vector_obj* args, int count);
sBlock* sBlock_new2(sBlock* block);
void sBlock_delete(sBlock* self);
void sBlock_save(sBlock* self, int fd);
sBlock* sBlock_load(int fd);

typedef struct
{
    int mKind; // 0:string 2: statments 4: statments2 5: block 6: process substituion
    void* mBody;
} sArg; // mArgs

sArg* sArg_new(int kind, void* body);
sArg* sArg_new2(sArg* arg);
void sArg_delete(sArg* arg);

void sArg_save(sArg* self, int fd);
sArg* sArg_load(int fd);

typedef struct
{
    enum eCommandKind mKind;
    vector_obj* mArgs;
        // vector of sArg*
        // ޥɥ饤
    vector_obj* mRedirects;
        // vector of sRedirect* 
        // 쥯Υꥹ

    void* mExtra;
} sCommand;

#ifdef MDEBUG 
void sCommand_view(sCommand* self);
#endif

sCommand* sCommand_new();
#define COMMAND_NEW() sCommand_new()

void sCommand_save(sCommand* self, int fd);
sCommand* sCommand_load(int fd);
void sCommand_delete(sCommand* self);

typedef struct
{
    vector_obj* mCommands;   // ޥ

    string_obj* mFName;      // ʸե̾
    int mLine;               // ʸ

    string_obj* mNotEvaled;  // run˥ѡʸ
                             // Ķѿȥޥޤʸ
    string_obj* mTitle;      // ȥ

    enum eStatmentTerminated mTerminated;   
            // ʣʸκǸ夬ǶڤƤ뤫

    BOOL mRCodeReverse;      // ꥿󥳡ɤȿž

    BOOL mBackground;        // Хå饦ɤǼ¹

    int mGlobalPipeIn;       // ХѥפϤ뤫 1:  4: ʥС
    int mGlobalPipeOut;      // ХѥפνϤ뤫 1:  4:ʥС
    int mGlobalPipeInNum;    // ʥСХѥפֹ
    int mGlobalPipeOutNum;   // ʥСХѥפֹ

    int mSTDINPipe;         // STDIN ѥפ뤫ʤ
} sStatment;

#ifdef MDEBUG
void sStatment_view(sStatment* self);
#endif

sStatment* sStatment_new();
#define STATMENT_NEW() sStatment_new()

sStatment* sStatment_new2(sStatment* statment);

void sStatment_delete(sStatment* self);
void sStatment_save(sStatment* self, int fd);
sStatment* sStatment_load(int fd);

sStatments* sStatments_new();
#define STATMENTS_NEW() sStatments_new()

void sStatments_view(sStatments* self);

void sStatments_save(sStatments* self, int fd);
sStatments* sStatments_load(int fd);
void sStatments_delete(sStatments* self);

typedef struct {
    string_obj* name;       // ޥ̾
    string_obj* arg_name;
    sStatments* statments; // ̿ɽ
} sFunction;

sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_name);
sFunction* sFunction_new2(sFunction* func);
  
void sFunction_delete(sFunction* self);
void sFunction_save(sFunction* self, int fd);
sFunction* sFunction_load(int fd);
void sFunction_view(sFunction* self);

typedef struct {
    pid_t mPid;
    string_obj* mTitle;
    BOOL mWriterProcess;
} sChildProgram;

sChildProgram* sChildProgram_new(pid_t pid, char* title, BOOL writer_process);
void sChildProgram_delete(sChildProgram* self);

typedef struct {
    string_obj* mName;
    fInnerCommand mFun;
} sInnerCommand;

sInnerCommand* sInnerCommand_new(char* name, fInnerCommand fun);
void sInnerCommand_delete(sInnerCommand* self);

void sigchld_block(int block);

////////////////////////////////////////////////////////////
/// from saphire_commands.c
////////////////////////////////////////////////////////////
int statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, int j, char* title, BOOL input, BOOL* return_, char* sname, int line);
BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers,char* sname, int line);

////////////////////////////////////////////////////////////
/// from saphire_parser.c
////////////////////////////////////////////////////////////
BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* line);
BOOL parse(char* cmdline, char* sname, int* line, sStatments* statments);
BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname, int* line);

typedef struct {
    int number;
    int number2;
    int number_num;
} sIndex;

sIndex* sIndex_new(int num, int num2, int num_num);
void sIndex_delete(sIndex* self);

BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment, BOOL line_inc, char* sname, int* line);

////////////////////////////////////////////////////////////
// from saphire_vm.c
////////////////////////////////////////////////////////////
BOOL forground_job(int num);
void background_job(int num);
void tmpfile_all_sweep();
void tmpfile_sweep();

extern vector_obj* gStackTraceFName;
extern vector_obj* gStackTraceLineNum;
    // åե졼εϿ

int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_);
    // saphire_compile2ǥѥ뤷ѥ̤¹Ԥ롣
    // verboseϼ¹Ԥ˼¹Ԥ륳ޥ̾ɽ뤫ɤ

#endif
